@@ -106,12 +106,8 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         if kwargs.get('data'):
             bgpvpn_data = kwargs.get('data')
         else:
-            bgpvpn = {'name': 'bgpvpn1',
-                      'type': 'l3',
-                      'route_targets': ['1234:56'],
-                      'tenant_id': self._tenant_id}
-            bgpvpn.update(kwargs)
-            bgpvpn_data = {'bgpvpn': bgpvpn}
+            bgpvpn_data = copy.copy(self.bgpvpn_data)
+            bgpvpn_data['bgpvpn'].update(kwargs)
         bgpvpn_req = self.new_create_request(
             'bgpvpn/bgpvpns', bgpvpn_data, fmt=fmt)
         res = bgpvpn_req.get_response(self.ext_api)
@@ -128,7 +124,6 @@ class BgpvpnTestCaseMixin(test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
         fmt = 'json'
         data = {'network_association': {'network_id': net_id,
                                         'tenant_id': self._tenant_id}}
-#         data = {'network_association': {'network_id': net_id}}
         bgpvpn_net_req = self.new_create_request(
             'bgpvpn/bgpvpns',
             data=data,
@@ -448,17 +443,42 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
     def setUp(self):
         super(TestBGPVPNServiceDriverDB, self).setUp()
 
+    def _raise_bgpvpn_driver_precommit_exc(self, *args, **kwargs):
+            raise extensions.bgpvpn.BGPVPNDriverError(
+                method='precommit method')
+
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'create_bgpvpn_postcommit')
+    @mock.patch.object(driver_api.BGPVPNDriver,
+                       'create_bgpvpn_precommit')
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'create_bgpvpn')
-    def test_create_bgpvpn(self, mock_create_db, mock_create_postcommit):
+    def test_create_bgpvpn(self, mock_create_db,
+                           mock_create_precommit,
+                           mock_create_postcommit):
         mock_create_db.return_value = self.converted_data['bgpvpn']
         with self.bgpvpn(do_delete=False):
             mock_create_db.assert_called_once_with(
                 mock.ANY, self.converted_data['bgpvpn'])
+            mock_create_precommit.assert_called_once_with(
+                mock.ANY, self.converted_data['bgpvpn'])
             mock_create_postcommit.assert_called_once_with(
                 mock.ANY, self.converted_data['bgpvpn'])
 
+    def test_create_bgpvpn_precommit_fails(self):
+        with mock.patch.object(driver_api.BGPVPNDriver,
+                               'create_bgpvpn_precommit',
+                               new=self._raise_bgpvpn_driver_precommit_exc):
+            # Assert that an error is returned to the client
+            bgpvpn_req = self.new_create_request(
+                'bgpvpn/bgpvpns', self.bgpvpn_data)
+            res = bgpvpn_req.get_response(self.ext_api)
+            self.assertEqual(webob.exc.HTTPError.code,
+                             res.status_int)
+
+            # Assert that no bgpvpn has been created
+            list = self._list('bgpvpn/bgpvpns', fmt='json')
+            self.assertEqual([], list['bgpvpns'])
+
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'delete_bgpvpn_postcommit')
     def test_delete_bgpvpn(self, mock_delete_postcommit):
@@ -507,11 +527,14 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
 
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'update_bgpvpn_postcommit')
+    @mock.patch.object(driver_api.BGPVPNDriver,
+                       'update_bgpvpn_precommit')
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb,
                        'update_bgpvpn')
-    def test_update_bgpvpn(self, mock_update_db, mock_update_postcommit):
+    def test_update_bgpvpn(self, mock_update_db,
+                           mock_update_precommit,
+                           mock_update_postcommit):
         with self.bgpvpn() as bgpvpn:
-            new_data = {"bgpvpn": {"name": "foo"}}
             old_bgpvpn = copy.copy(self.bgpvpn_data['bgpvpn'])
             old_bgpvpn['id'] = bgpvpn['bgpvpn']['id']
             old_bgpvpn['networks'] = []
@@ -521,19 +544,40 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
 
             mock_update_db.return_value = new_bgpvpn
 
+            new_data = {"bgpvpn": {"name": "foo"}}
             self._update('bgpvpn/bgpvpns',
                          bgpvpn['bgpvpn']['id'],
                          new_data)
 
             mock_update_db.assert_called_once_with(
                 mock.ANY, bgpvpn['bgpvpn']['id'], new_data['bgpvpn'])
+            mock_update_precommit.assert_called_once_with(
+                mock.ANY, old_bgpvpn, new_bgpvpn)
             mock_update_postcommit.assert_called_once_with(
                 mock.ANY, old_bgpvpn, new_bgpvpn)
 
+    def test_update_bgpvpn_precommit_fails(self):
+        with self.bgpvpn() as bgpvpn, \
+            mock.patch.object(driver_api.BGPVPNDriver,
+                              'update_bgpvpn_precommit',
+                              new=self._raise_bgpvpn_driver_precommit_exc):
+                new_data = {"bgpvpn": {"name": "foo"}}
+                self._update('bgpvpn/bgpvpns',
+                             bgpvpn['bgpvpn']['id'],
+                             new_data,
+                             expected_code=webob.exc.HTTPError.code)
+                show_bgpvpn = self._show('bgpvpn/bgpvpns',
+                                         bgpvpn['bgpvpn']['id'])
+                self.assertEqual(self.bgpvpn_data['bgpvpn']['name'],
+                                 show_bgpvpn['bgpvpn']['name'])
+
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'create_net_assoc_postcommit')
+    @mock.patch.object(driver_api.BGPVPNDriver,
+                       'create_net_assoc_precommit')
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'create_net_assoc')
     def test_create_bgpvpn_net_assoc(self, mock_db_create_assoc,
+                                     mock_pre_commit,
                                      mock_post_commit):
         with self.bgpvpn() as bgpvpn:
             bgpvpn_id = bgpvpn['bgpvpn']['id']
@@ -550,9 +594,35 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
                                     do_disassociate=False):
                     mock_db_create_assoc.assert_called_once_with(
                         mock.ANY, bgpvpn_id, data)
+                    mock_pre_commit.assert_called_once_with(mock.ANY,
+                                                            net_assoc_dict)
                     mock_post_commit.assert_called_once_with(mock.ANY,
                                                              net_assoc_dict)
 
+    def test_create_bgpvpn_net_assoc_precommit_fails(self):
+        with self.bgpvpn() as bgpvpn, \
+            self.network() as net, \
+            mock.patch.object(driver_api.BGPVPNDriver,
+                              'create_net_assoc_precommit',
+                              new=self._raise_bgpvpn_driver_precommit_exc):
+            fmt = 'json'
+            data = {'network_association': {'network_id': net['network']['id'],
+                                            'tenant_id': self._tenant_id}}
+            bgpvpn_net_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='network_associations')
+            res = bgpvpn_net_req.get_response(self.ext_api)
+            # Assert that driver failure returns an error
+            self.assertEqual(webob.exc.HTTPError.code,
+                             res.status_int)
+            # Assert that the bgpvpn is not associated to network
+            bgpvpn_new = self._show('bgpvpn/bgpvpns',
+                                    bgpvpn['bgpvpn']['id'])
+            self.assertEqual([], bgpvpn_new['bgpvpn']['networks'])
+
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_net_assoc')
     def test_get_bgpvpn_net_assoc(self, mock_get_db):
         with self.bgpvpn() as bgpvpn:
@@ -605,26 +675,55 @@ class TestBGPVPNServiceDriverDB(BgpvpnTestCaseMixin):
 
     @mock.patch.object(driver_api.BGPVPNDriver,
                        'create_router_assoc_postcommit')
+    @mock.patch.object(driver_api.BGPVPNDriver,
+                       'create_router_assoc_precommit')
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'create_router_assoc')
     def test_create_bgpvpn_router_assoc(self, mock_db_create_assoc,
+                                        mock_pre_commit,
                                         mock_post_commit):
-        with self.bgpvpn() as bgpvpn:
+        with self.bgpvpn() as bgpvpn, \
+            self.router(tenant_id=self._tenant_id) as router:
             bgpvpn_id = bgpvpn['bgpvpn']['id']
-            with self.router(tenant_id=self._tenant_id) as router:
-                router_id = router['router']['id']
-                assoc_id = _uuid()
-                data = {'tenant_id': self._tenant_id,
-                        'router_id': router_id}
-                router_assoc_dict = copy.copy(data)
-                router_assoc_dict.update({'id': assoc_id,
-                                          'bgpvpn_id': bgpvpn_id})
-                mock_db_create_assoc.return_value = router_assoc_dict
-                with self.assoc_router(bgpvpn_id, router_id=router_id,
-                                       do_disassociate=False):
-                    mock_db_create_assoc.assert_called_once_with(
-                        mock.ANY, bgpvpn_id, data)
-                    mock_post_commit.assert_called_once_with(mock.ANY,
-                                                             router_assoc_dict)
+            router_id = router['router']['id']
+            assoc_id = _uuid()
+            data = {'tenant_id': self._tenant_id,
+                    'router_id': router_id}
+            router_assoc_dict = copy.copy(data)
+            router_assoc_dict.update({'id': assoc_id,
+                                      'bgpvpn_id': bgpvpn_id})
+            mock_db_create_assoc.return_value = router_assoc_dict
+            with self.assoc_router(bgpvpn_id, router_id=router_id,
+                                   do_disassociate=False):
+                mock_db_create_assoc.assert_called_once_with(
+                    mock.ANY, bgpvpn_id, data)
+                mock_pre_commit.assert_called_once_with(mock.ANY,
+                                                        router_assoc_dict)
+                mock_post_commit.assert_called_once_with(mock.ANY,
+                                                         router_assoc_dict)
+
+    def test_create_bgpvpn_router_assoc_precommit_fails(self):
+        with self.bgpvpn() as bgpvpn, \
+                self.router(tenant_id=self._tenant_id) as router, \
+                mock.patch.object(driver_api.BGPVPNDriver,
+                                  'create_router_assoc_precommit',
+                                  new=self._raise_bgpvpn_driver_precommit_exc):
+            fmt = 'json'
+            data = {'router_association': {'router_id': router['router']['id'],
+                                           'tenant_id': self._tenant_id}}
+            bgpvpn_router_req = self.new_create_request(
+                'bgpvpn/bgpvpns',
+                data=data,
+                fmt=fmt,
+                id=bgpvpn['bgpvpn']['id'],
+                subresource='router_associations')
+            res = bgpvpn_router_req.get_response(self.ext_api)
+            # Assert that driver failure returns an error
+            self.assertEqual(webob.exc.HTTPError.code,
+                             res.status_int)
+            # Assert that the bgpvpn is not associated to network
+            bgpvpn_new = self._show('bgpvpn/bgpvpns',
+                                    bgpvpn['bgpvpn']['id'])
+            self.assertEqual([], bgpvpn_new['bgpvpn']['routers'])
 
     @mock.patch.object(bgpvpn_db.BGPVPNPluginDb, 'get_router_assoc')
     def test_get_bgpvpn_router_assoc(self, mock_get_db):
